﻿"use strict";
// External functions: (should be set on External object before importing this script)
//   import(string file): takes a file string argument, loads external script, returns [ function, hash ] or [ false, reason ] if failed
//   md5Hash(string file): takes a file string argument, returns [ true, hash ] or [ false, reason ] if failed
//
// Other stuff:
//   console.writeLine: like C# Console.WriteLine.
//   debug.writeLine: like C# Debug.WriteLine.

var ModuleInfo = Class(function (out) {
    out.constructor = function ModuleInfo(file, hash, namespace) {
        this.file = file;
        this.hash = hash;
        this.namespace = namespace;
    }

    return out;
}({}));

var Namespace = Class(function (out) {
    out.constructor = function Namespace(name) {
        Object.defineProperty(this, "__namespace", { value: name, writable: false });
        Object.defineProperty(this, "__instances", { value: new Collections.Dictionary(), writable: false });
    };

    out.createInstance = function createInstance(id) {
        var inst = Object.create(this);
        this.__instances.set(id, inst);
        inst.toString = function toString() { return "Instance:" + this.__namespace; };
        Object.defineProperty(inst, "__id", { value: id, writeable: false, configurable: false });
        return inst;
    };

    out.removeInstance = function removeInstance(id) {
        this.__instances.remove(id);
    }

    out.toString = mutable(function toString() {
        return "Namespace:" + this.__namespace;
    });

    return out;
}({}));

Error.getMessage = function getMessage(e) {
    if (!e) {
        return "";
    }
    if (typeof e === "string") {
        return e;
    }
    else if (e.message) {
        return e.message;
    }
    return "";
}

this.DonkeyGlobal = (function (global) {
    var module = {};

    var modules = new Collections.Dictionary(String, ModuleInfo);
    module.modules = modules;

    var namespaces = new Collections.Dictionary(String, Namespace);
    module.namespaces = namespaces;

    module.unsafeImport = function unsafeImport(file) {
        console.writeLine("Importing script " + file + "...");
        var ret = External.import(file);

        if (ret[0] === false) {
            throw new Error(ret[1]);
        }

        var mod = ret[0]();
        modules.set(file, new ModuleInfo(file, ret[1], mod));
        console.writeLine("Imported script " + file + " into namespace " + mod.__namespace);
        return mod;
    }

    module.importScript = function importScript(file, unsafe) {
        if (typeof unsafe === "undefined") {
            unsafe = false;
        }

        var mod;

        if (unsafe) {
            mod = module.unsafeImport(file);
        }
        else {
            try {
                mod = module.unsafeImport(file);
            }
            catch (e) {
                console.writeLine("Failed to load script " + file + ": " + Error.getMessage(e));
            }
        }

        return mod;
    }
    global.importScript = module.importScript;

    var reloadKeys = null;
    var keyIndex = 0;

    module.reloadModules = function reloadModules() {
        if (reloadKeys == null) {
            reloadKeys = modules.keys;
        }

        if (keyIndex >= reloadKeys.length) {
            keyIndex = 0;
            reloadKeys = null;
            return;
        }

        var key = reloadKeys[keyIndex];
        var info = modules.get(key);
        var ret = External.md5Hash(info.file);

        if (ret[0]) {
            var newHash = ret[1];

            if (info.hash != newHash) {
                module.importScript(info.file);
                info.hash = newHash;
            }
        }

        keyIndex++;
    }

    global.namespace = function namespace(name) {
        var result = {};
        if (namespaces.tryGet(name, result)) {
            return result.value;
        }
        else {
            var ns = new Namespace(name);
            namespaces.set(name, ns);
            global[name] = ns;
            return ns;
        }
    }

    return module;
}(this));
