﻿(function () {
    var ns = namespace("Map");

    ns.start = function start(time) {
        if (typeof this.FloorNumber === "number" && typeof this.MapNumber === "number") {
            this.Hud.FloorNumber = this.FloorNumber;
            this.Hud.MapNumber = this.MapNumber;
        }

        var self = this;
        this.startTime = time;
        this.entities = new Collections.List(Entities.Entity);
        this.particles = new Collections.List(Particles.Particle);

        // create player
        /*this.player = new Entities.Player(this, 6, 6);
        this.player.onStoppedMoving = function (p) { self.playerMoved.call(self); };

        this.addEntity(this.player);
        this.addEntity(new Entities.Key(this, 6, 5));
        this.addEntity(new Entities.Door(this, 6, 7));*/

        var objects = this.Level.GetObjects();
        for (var i = 0; i < objects.Length; i++) {
            var obj = objects[i];
            var ent = null;
            switch (obj.EntityType) {
                case Constants.EntPlayer:
                    ent = new Entities.Player(this, obj.ID, obj.TileX, obj.TileY);
                    this.player = ent;
                    break;
                case Constants.EntKey:
                    ent = new Entities.Key(this, obj.ID, obj.TileX, obj.TileY);
                    break;
                case Constants.EntDoor:
                    ent = new Entities.Door(this, obj.ID, obj.TileX, obj.TileY);
                    break;
                case Constants.EntHatA:
                    ent = new Entities.HatA(this, obj.ID, obj.TileX, obj.TileY);
                    break;
                case Constants.EntHatB:
                    ent = new Entities.HatB(this, obj.ID, obj.TileX, obj.TileY);
                    break;
                case Constants.EntHatC:
                    ent = new Entities.HatC(this, obj.ID, obj.TileX, obj.TileY);
                    break;
                case Constants.EntHatD:
                    ent = new Entities.HatD(this, obj.ID, obj.TileX, obj.TileY);
                    break;
                case Constants.EntGuestA:
                    ent = new Entities.Guest(this, obj.ID, 0, obj.TileX, obj.TileY);
                    ent.animStart = this.Random.NextDouble();
                    break;
                case Constants.EntGuestB:
                    ent = new Entities.Guest(this, obj.ID, 1, obj.TileX, obj.TileY);
                    ent.animStart = this.Random.NextDouble();
                    break;
                case Constants.EntGuestC:
                    ent = new Entities.Guest(this, obj.ID, 2, obj.TileX, obj.TileY);
                    ent.animStart = this.Random.NextDouble();
                    break;
                case Constants.EntGuestD:
                    ent = new Entities.Guest(this, obj.ID, 3, obj.TileX, obj.TileY);
                    ent.animStart = this.Random.NextDouble();
                    break;
            }
            if (ent) {
                this.addEntity(ent);
            }
        }

        this.player.onStoppedMoving = function (p) { self.playerMoved.call(self); };
        this.waitingForInput = true;
        this.enemyTurn = false;
        this.playerTurn = true;
    }

    ns.canMoveTo = function canMoveTo(src, x, y) {
        if (!this.Level.CanPass(x, y)) {
            return false;
        }
        var ret = true;
        for (var i = this.entities.count - 1; i >= 0; i--) {
            var target = this.entities.get(i);
            if (target.tileX == x && target.tileY == y && !target.canPass(src)) {
                ret = false;
            }
        }
        return ret;
    }

    ns.playerMoved = function playerMoved() {
        /*for (var i = 0; i < this.entities.count; i++) {
            var ent = this.entities.get(i);
            if (ent.isPickUp && ent.tileX == this.player.tileX && ent.tileY == this.player.tileY) {
                ent.pickedUp(this.player);
                this.removeEntity(ent);
            }
        }*/

        this.enemyTakeTurn();
    }

    ns.pause = function pause(duration) {
        this.paused = true;
        this.pauseStart = this.time;
        this.pauseEnd = this.pauseStart + duration;
    }

    ns.startEnemyTurn = function startEnemyTurn() {
        this.enemyTurn = true;
        this.turnStage = 0;
        this.entityIndex = 0;
        //console.writeLine("Spluffs' turn (reduce KO counter)");
    }

    ns.anyKey = function anyKey(time) {
        if (this.finished && time > this.sceneEndTime) {
            console.writeLine("Level end");
            this._nextScene = true;
        }
    }

    ns.finish = function finish() {
        this.finished = true;
        this.finishedTime = this.time;
        this.sceneEndTime = this.time + 3.5;
        this.Game.PlayMusic("victory", false);
    }

    ns.enemyTakeTurn = function enemyTakeTurn() {
        if (this.finished || this._nextScene) {
            return;
        }

        if (this.paused) {
            if (this.time < this.pauseEnd) {
                return;
            }
            this.paused = false;
        }

        if (this.turnStage == 0) {
            while (this.entityIndex < this.entities.count) {
                var ent = this.entities.get(this.entityIndex);
                this.entityIndex++;
                if (ent.reduceKoCounter) {
                    ent.reduceKoCounter();
                    if (this.paused) {
                        return;
                    }
                }
            }

            this.turnStage = 1;
            this.entityIndex = 0;
            //console.writeLine("Spluffs' turn (proper)");
        }

        if (this.turnStage == 1) {
            while (this.entityIndex < this.entities.count) {
                var ent = this.entities.get(this.entityIndex);
                this.entityIndex++;
                if (ent.takeTurn) {
                    ent.takeTurn(this.player);
                    if (this.paused) {
                        return;
                    }
                }
            }

            this.turnStage = 2;
            this.entityIndex = 0;
        }

        this.enemyTurn = false;
        this.playerTurn = true;
    }

    ns.removeEntity = function removeEntity(ent) {
        this.entities.remove(ent);
        this.Level.RemoveEntity(ent);
    }

    ns.addEntity = function addEntity(ent) {
        this.entities.add(ent);
        this.Level.AddEntity(ent);
    }

    ns.addParticle = function addParticle(part) {
        this.particles.add(part);
        this.Level.AddParticle(part);
    }

    ns.removeParticle = function removeParticle(part) {
        this.particles.remove(part);
        this.Level.RemoveParticle(part);
    }

    ns.move = function move(direction, type, time) {
        if (!this.waitingForInput || this.enemyTurn || this._nextScene || this.finished) {
            return;
        }

        var toX = this.player.tileX;
        var toY = this.player.tileY;
        if (direction == Constants.DirDown) {
            toY++;
        }
        else if (direction == Constants.DirUp) {
            toY--;
        }
        else if (direction == Constants.DirLeft) {
            toX--;
        }
        else if (direction == Constants.DirRight) {
            toX++;
        }

        if (this.canMoveTo(this.player, toX, toY)) {
            this.player.time = time;
            this.player.move(direction);
            this.waitingForInput = false;
            this.startEnemyTurn();
        }
        else {
            this.player.facing = direction;
            for (var i = 0; i < this.entities.count; i++) {
                var ent = this.entities.get(i);
                if (ent.tileX == toX && ent.tileY == toY && ent.hat == this.player.hat && this.player.hat > -1 && !ent.ko && !ent.locked) {
                    ent.move(direction, 5);
                    this.player.hat = -1;
                    this.player.hatTexture = null;
                    this.addParticle(new Particles.ExplosionEmitter(this, 0.3, this.player.tileX, this.player.tileY - 0.4, 0.15, 0.15));
                    this.startEnemyTurn();
                    this.Game.PlaySound("moveplz2");
                }
            }
        }
    }

    ns.update = function update(time) {
        this.time = time;

        if (this.finished) {
            if (this.time >= this.sceneEndTime) {
                this.Hud.pressAnyKey = true;
                this.player.curAnim = "stand";
                this.player.facing = Constants.DirDown;
            }
        }

        if (this.waitingForInput && this.player.curAnim == "walk") {
            this.player.curAnim = "stand";
        }

        var acting = false;

        for (var i = 0; i < this.entities.count; i++) {
            var ent = this.entities.get(i);
            ent.time = time;
            ent.update(time);
            acting = acting || ent.acting;
        }

        for (var i = this.particles.count - 1; i >= 0; i--) {
            var part = this.particles.get(i);
            part.time = time;
            part.update(time);
            if (part.dead) {
                this.particles.removeAt(i);
                this.Level.RemoveParticle(part);
            }
        }
        
        if (!acting) {
            if (this.enemyTurn) {
                this.enemyTakeTurn();
            }
            else if (this.playerTurn) {
                //console.writeLine("Player's turn");
                this.playerTurn = false;
                this.waitingForInput = true;
            }
        }
    }

    ns.render = function render(batch, time) {
        this.Game.GraphicsDevice.Clear(new Color(0, 0, 0));
        this.Level.SetOffset(this.player.tileX + this.player.offsetX, this.player.tileY + this.player.offsetY);
        this.Level.Render(batch);
    }

    return ns;
}());