﻿(function () {
    var ns = namespace("Particles");

    var id = 0;

    var Particle = Class(function (out) {
        out.constructor = function Particle(anim, facing, duration, x, y) {
            this.id = ++id;
            this.anim = anim;
            this.facing = facing;
            this.x = x;
            this.y = y;
            this.duration = duration;
            this.fx = 0;
        }

        out.update = mutable(function update() {
            if (!this.start) {
                this.start = this.time;
            }

            if (this.anim) {
                var frame = this.anim.getFrame(this.curAnim || "default", this.facing, this.time - this.start);
                this.texture = frame.texture;
            }

            if (this.duration > 0 && this.time >= this.start + this.duration) {
                this.dead = true;
            }
        });

        return out;
    }({}));
    ns.Particle = Particle;

    var Punch = Class(Particle, function (out) {
        out.constructor = function Punch(facing, x, y) {
            Particle.call(this, Animations.punch, facing, Animations.punch.duration, x, y);
        }

        return out;
    }({}));
    ns.Punch = Punch;

    var KoStars = Class(Particle, function (out) {
        out.constructor = function KoStars(x, y, duration) {
            Particle.call(this, Animations.koStars, Engine.DirDown, duration || 0, x, y - 1.0 / 16);
        }

        return out;
    }({}));
    ns.KoStars = KoStars;

    var KoCount = Class(Particle, function (out) {
        out.constructor = function KoCount(x, y) {
            Particle.call(this, Animations.koCount, Engine.DirDown, 0, x, y);
        }

        out.setCounter = function setCounter(count) {
            console.writeLine("setCounter " + count);
            this.curAnim = "" + count;
        }

        return out;
    }({}));
    ns.KoCount = KoCount;

    var Explosion = Class(Particle, function (out) {
        out.constructor = function Explosion(type, fx, x, y) {
            Particle.call(this, Animations.explosions[type], Engine.DirDown, Animations.explosions[type].duration, x, y);
            this.fx = fx;
        }

        return out;
    }({}));
    ns.Explosion = Explosion;

    var ExplosionEmitter = Class(Particle, function (out) {
        out.constructor = function ExplosionEmitter(map, duration, x, y, hSpread, vSpread) {
            Particle.call(this, null, Engine.DirDown, duration, x, y);
            this.map = map;
            this.lastParticleTime = 0;
            this.interval = 0.05;

            if (!hSpread) {
                this.hSpread = 0.5;
            }
            else {
                this.hSpread = hSpread;
            }
            if (!vSpread) {
                this.vSpread = 0.5;
            }
            else {
                this.vSpread = vSpread;
            }
        }

        out.update = mutable(function update() {
            base(this).update.call(this);

            if (!this.dead) {
                if (this.lastParticleTime + this.interval < this.time) {
                    this.lastParticleTime = this.time;
                    this.map.addParticle(
                        new Explosion(this.map.Random.Next(Animations.explosions.length), this.map.Random.Next(4),
                            this.x - (this.hSpread / 2) + this.map.Random.NextDouble() * this.hSpread,
                            this.y - (this.vSpread / 2) + this.map.Random.NextDouble() * this.vSpread));
                }
            }
        });

        return out;
    }({}));
    ns.ExplosionEmitter = ExplosionEmitter;

    return ns;
}());